/****************************************************************************
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *~                             IR Kernel                                    ~
 *~                         Copyright(C) 1993                                ~
 *~                       Software Kinetics Ltd.                             ~
 *~                        All Rights Reserved                               ~
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Copyright Software Kinetics Ltd.
 * 
 * Software Kinetics Ltd. has granted a licence of this software to
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve,
 * reproduce and sell the software.
 *
 * The software is provided to the user "as is" without warranty of
 * any kind.  Neither Software Kinetics Ltd. nor the National Library
 * of Canada warrants,guarantees or makes any representation regarding
 * the use, or the results of the use of the software in terms of
 * accuracy,reliability, or otherwise.  The entire risk as results and
 * performance of the software is assumed by the user. Neither
 * Software Kinetics Ltd. nor the National Library of Canada shall be
 * liable for any direct, indirect, consequential or incidental
 * damages arising out the use of software.
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
 *
 * Program: irckinit.c
 *
 * Purpose: This file contains the routines to check the mandatory fields of
 *          init request PDU and init response PDU.
 *
 * Invocation:
 *
 * Input Arguments:
 *
 * Output/Exit Values: None
 *
 * Functions used:
 *
 * Version: Date:   94/05/24
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference: IR KERNEL - TARGET PRELIMINARY SOFTWARE DESIGN DOCUMENT
 *            IR KERNEL - TARGET SOFTWARE DETAIL DESIGN DOCUMENT
 *            IR KERNEL - INTERFACE REQUIREMENTS SPECIFICATION
 *            OSI - ANSI Z39.50 DOCUMENTATION
 *
 * Notes:
 *
 * $Id: tpinit.c,v 1.3 1994/02/11 13:37:08 irsuprt Exp $
 * $Log: tpinit.c,v $
 * Initial revision
 *
 *
 *****************************************************************************/

#include <stdio.h>
#include <rpc/rpc.h>
#include <fcntl.h>
#include <pwd.h>
#include <ctype.h>
#include <sys/errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include "ir.h"
#include "asn_incl.h"
#include "irerror.h"
#include "irconst.h"
#include "common.h"

/**************************** Global Ids *********************************/

char OID_CLASS_ACCESS_CONTROL_ENCODED[35];

/**************************** Extern Global Ids **************************/

extern int errno;

/*****************************************************************************
 *
 * Function: ir_fill_init_response
 *
 * Purpose: This function fills and creates the init response PDU.
 *
 * Input Arguments:
 *              PDU_ireq - init request PDU pointer
 *              accept - 1 (init response filled with Accept)
 *                       0 (init response filled with reject)
 *              protocol_version - replied protocol version
 *              options - replied options
 *
 * Return values: SUCCESS or FAILED
 *
 * Output Arguments:
 *              PDU_iresp - init response PDU pointer
 *
 * External Variables:
 *
 * Function used:
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference:
 *
 * Notes:
 *
 ****************************************************************************/
ir_fill_init_response(PDU_ireq, accept, protocol_version, options, PDU_iresp)
    APDU *PDU_ireq;
    int accept;
    char protocol_version;
    char options;
    APDU *PDU_iresp;
{

#ifdef DEBUG_LEVEL1
    fprintf(DEBUG_FILE1, "Calling ir_fill_init_response\n");
#endif

    PDU_iresp->a.initResp = Asn1Alloc(sizeof(struct InitializeResp));

    PDU_iresp->choiceId = APDU_INITRESP;

    PDU_iresp->a.initResp->referenceId.octetLen = PDU_ireq->a.initReq->referenceId.octetLen ;
    PDU_iresp->a.initResp->referenceId.octs = Asn1Alloc(PDU_ireq->a.initReq->referenceId.octetLen);	
    memcpy(PDU_iresp->a.initResp->referenceId.octs, PDU_ireq->a.initReq->referenceId.octs , PDU_ireq->a.initReq->referenceId.octetLen);

    PDU_iresp->a.initResp->protocolVersion.bitLen = 1;
    PDU_iresp->a.initResp->protocolVersion.bits = Asn1Alloc(1);
    PDU_iresp->a.initResp->protocolVersion.bits[0] = protocol_version;

    PDU_iresp->a.initResp->options.bitLen = 1;
    PDU_iresp->a.initResp->options.bits = Asn1Alloc(1);
    /* support 0-search, 1-present, 2-delete, 3-resource report, */
    /* 4-trig resc ctrl, 5-resc ctrl, 6-access control. */
    PDU_iresp->a.initResp->options.bits[0] = options;

    PDU_iresp->a.initResp->preferredMessageSize = MAXMESSAGESIZE;
    PDU_iresp->a.initResp->maximumRecordSize = MAXMESSAGESIZE;
    if (accept)
        PDU_iresp->a.initResp->result = TRUE;        /* accept */
    else
        PDU_iresp->a.initResp->result = FALSE;        /* reject */

    PDU_iresp->a.initResp->implementationId.octetLen = sizeof(IMPLEMENTATION_ID) - 1;
    PDU_iresp->a.initResp->implementationId.octs = (char *) Asn1Alloc(PDU_iresp->a.initResp->implementationId.octetLen);
    memcpy(PDU_iresp->a.initResp->implementationId.octs, IMPLEMENTATION_ID, PDU_iresp->a.initResp->implementationId.octetLen);

    PDU_iresp->a.initResp->implementationName.octetLen = sizeof(IMPLEMENTATION_NAME) - 1;
    PDU_iresp->a.initResp->implementationName.octs = (char *) Asn1Alloc(PDU_iresp->a.initResp->implementationName.octetLen);
    memcpy(PDU_iresp->a.initResp->implementationName.octs, IMPLEMENTATION_NAME, PDU_iresp->a.initResp->implementationName.octetLen);

    PDU_iresp->a.initResp->implementationVersion.octetLen = sizeof(IMPLEMENTATION_VERSION) - 1;
    PDU_iresp->a.initResp->implementationVersion.octs = (char *) Asn1Alloc(PDU_iresp->a.initResp->implementationVersion.octetLen);
    memcpy(PDU_iresp->a.initResp->implementationVersion.octs, IMPLEMENTATION_VERSION, PDU_iresp->a.initResp->implementationVersion.octetLen);

    return (SUCCESS);

}

/*****************************************************************************
 *
 * Function: ir_create_init_response
 *
 * Purpose: This function creates the init response PDU, encodes, and
 *          creates the init response APDU for sending to Origin.
 *
 * Input Arguments:
 *              PDU_ireq - init request PDU
 *              accept - 1 (init response filled with Accept)
 *                       0 (init response filled with reject)
 *              protocol_version - replied protocol version
 *              options - replied options
 *              data_ptr - temporary data buffer for encoding process
 *
 * Return values: SUCCESS or FAILED
 *
 * Output Arguments:
 *              outputBuf - encoded APDU pointer
 *              APDU_length - the size of the encoded APDU
 *
 * External Variables:
 *
 * Function used:
 *              ir_fill_init_response()
 *              SBufInstallData()
 *              SBufResetInWriteRvsMode()
 *              BEncAPDU()
 *              ir_fwrite()
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference:
 *
 * Notes:
 *
 ****************************************************************************/
ir_create_init_response(PDU_ireq, accept, protocol_version, options, data_ptr, outputBuf, APDU_length)
	APDU *PDU_ireq;
    int accept;                        /* boolean */
    char protocol_version;
    char options;
    char **data_ptr;
    SBuf *outputBuf;
    int *APDU_length;
{
    APDU PDU_iresp;

#ifdef DEBUG_LEVEL1
    fprintf(DEBUG_FILE1, "Calling ir_create_init_response()\n");
#endif

    ir_fill_init_response(PDU_ireq, accept, protocol_version, options, &PDU_iresp);

    SBufInstallData(outputBuf, (*data_ptr), MAXMESSAGESIZE * 2);
    SBufResetInWriteRvsMode(outputBuf);

    *APDU_length = BEncAPDU(outputBuf, &PDU_iresp);

	if (SBufWriteError(outputBuf))
	{
		display_error("KERNEL", ERR_ENCODE_APDU, -1);
		return(FAILED);
	}

#ifdef DEBUG_LEVEL2
    fprintf(DEBUG_FILE2, "Init Response APDU Dump from tpinit\n");
    ir_fwrite(SBufDataPtr(outputBuf), SBufDataLen(outputBuf), 1, DEBUG_FILE2);
#endif

    return (SUCCESS);
}

/*****************************************************************************
 *
 * Function: ir_actual_check_init_request
 *
 * Purpose: This function checks the init request PDU to make sure the
 *          mandatory fields are present and valid.
 *
 * Input Arguments: PDU_ptr - init request PDU pointer
 *
 * Return values: SUCCESS or FAILED
 *
 * Output Arguments:
 *              protocol_version - final protocol version
 *              options - final options
 *
 * External Variables:
 *
 * Function used:
 *              display_error()
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference:
 *
 * Notes:
 *
 ****************************************************************************/
ir_actual_check_init_request(PDU_ptr, protocol_version, options)
    APDU *PDU_ptr;
    char *protocol_version;
    char *options;
{

#ifdef DEBUG_LEVEL1
    fprintf(DEBUG_FILE1, "Calling ir_actual_check_init_request\n");
#endif

#ifdef DEBUG_LEVEL3
    fprintf(DEBUG_FILE3, "ir_actual_check_init_request: referenceId.octetLen=%d\n", PDU_ptr->a.initReq->referenceId.octetLen);
    fprintf(DEBUG_FILE3, "ir_actual_check_init_request: referenceId.octs=%s/\n", PDU_ptr->a.initReq->referenceId.octs);
#endif

    /* checking the mandatory fields */

#ifdef DEBUG_LEVEL3
    fprintf(DEBUG_FILE3, "ir_actual_check_init_request: protocolVersion.bitLen=%d\n", PDU_ptr->a.initReq->protocolVersion.bitLen);
    fprintf(DEBUG_FILE3, "ir_actual_check_init_request: protocolVersion.bits[0]=%x\n", PDU_ptr->a.initReq->protocolVersion.bits[0] & 0xff);
#endif

    if ((PDU_ptr->a.initReq->protocolVersion.bitLen <= 0) ||
            (PDU_ptr->a.initReq->protocolVersion.bits[0] & 0xff <= 0)
        )
    {
        display_error("KERNEL", ERR_EMPTY_INIT_PROTOCOL_VERSION, -1);
        return (FAILED);
    }

    if (PDU_ptr->a.initReq->preferredMessageSize <= 0)
    {
#ifdef DEBUG_LEVEL3
		fprintf(DEBUG_FILE3, "ir_actual_check_init_request: preferredMessageSize=%ld\n", PDU_ptr->a.initReq->preferredMessageSize); 
#endif
        display_error("KERNEL", ERR_EMPTY_INIT_PREFERRED_MESSAGE_SIZE, -1);
        return (FAILED);
    }

    if (PDU_ptr->a.initReq->maximumRecordSize <= 0)
    {
        display_error("KERNEL", ERR_EMPTY_INIT_MAXIMUM_RECORD_SIZE, -1);
        return (FAILED);
    }

    /* end of checking the mandatory fields */

    /*
     * support 0-search 80H, 1-present 40H, 2-delete 20H, 3-resource report
     * 10H, 4-trig resc ctrl 08H, 5-resc ctrl 04H, 6-access control 02H. The
     * target process will downgrade to SR if the client is a SR client ie.
     * access control and resource control are "will support" not "will use".
     * In another words, the access control and resource control are not
     * enforced.
     */

#ifdef DEBUG_LEVEL3
    fprintf(DEBUG_FILE3, "ir_actual_check_init_request: INIT REQUEST OPTIONS from client: %x\n", (unsigned char) PDU_ptr->a.initReq->options.bits[0]);

    if ((PDU_ptr->a.initReq->options.bits[0] & 0x80) == 0x80)
		fprintf(DEBUG_FILE3, "ir_actual_check_init_request: Search option is on\n");
	else
		fprintf(DEBUG_FILE3, "ir_actual_check_init_request: Search option is off\n");
    if ((PDU_ptr->a.initReq->options.bits[0] & 0x40) == 0x40)
		fprintf(DEBUG_FILE3, "ir_actual_check_init_request: Present option is on\n");
	else
		fprintf(DEBUG_FILE3, "ir_actual_check_init_request: Present option is off\n");
    if ((PDU_ptr->a.initReq->options.bits[0] & 0x20) == 0x20)
		fprintf(DEBUG_FILE3, "ir_actual_check_init_request: Delete option is on\n");
	else
		fprintf(DEBUG_FILE3, "ir_actual_check_init_request: Delete option is off\n");
    if ((PDU_ptr->a.initReq->options.bits[0] & 0x10) == 0x10)
		fprintf(DEBUG_FILE3, "ir_actual_check_init_request: Resource Report option is on\n");
	else
		fprintf(DEBUG_FILE3, "ir_actual_check_init_request: Resource Report option is off\n");
    if ((PDU_ptr->a.initReq->options.bits[0] & 0x8) == 0x8)
		fprintf(DEBUG_FILE3, "ir_actual_check_init_request: Trigger Resource Control option is on\n");
	else
		fprintf(DEBUG_FILE3, "ir_actual_check_init_request: Trigger Resource Control option is off\n");
    if ((PDU_ptr->a.initReq->options.bits[0] & 0x4) == 0x4)
		fprintf(DEBUG_FILE3, "ir_actual_check_init_request: Resource Control option is on\n");
	else
		fprintf(DEBUG_FILE3, "ir_actual_check_init_request: Resource Control option is off\n");
    if ((PDU_ptr->a.initReq->options.bits[0] & 0x2) == 0x2)
		fprintf(DEBUG_FILE3, "ir_actual_check_init_request: Access Control option is on\n");
	else
		fprintf(DEBUG_FILE3, "ir_actual_check_init_request: Access Control option is off\n");

    fprintf(DEBUG_FILE3, "ir_actual_check_init_request: INIT REQUEST PROTOCOL VERSION from client =%x\n", (unsigned char) PDU_ptr->a.initReq->protocolVersion.bits[0]);
#endif

#ifdef ENFORCE_ACCESS_RESOURCE_CONTROL

#ifdef DEBUG_LEVEL1
	fprintf(DEBUG_FILE1, "IR TARGET ENFORCES ACCESS/RESOURCE CONTROL\n");
#endif

    /* if IR protocol bit is not set */
    if ((PDU_ptr->a.initReq->protocolVersion.bits[0] & IR_PROTOCOL_BIT) == 0)
    {
        display_error("KERNEL", ERR_INVALID_INIT_PROTOCOL_VERSION, -1);
        return (FAILED);
    }

    /* if access control and resource control option bits are not set */
    if (((PDU_ptr->a.initReq->options.bits[0] & ACCESS_CONTROL_BIT) == 0) || ((PDU_ptr->a.initReq->options.bits[0] & RESOURCE_CONTROL_BIT) == 0))
    {
        display_error("KERNEL", ERR_INVALID_INIT_REQUEST_OPTIONS, -1);
        return (FAILED);
    }

    *protocol_version = (char) 0xc0;        /* SR+IR */
    *options = (char) 0xfe;                /* all on */
    ACCESS_CONTROL_REQUIRED = TRUE;
    RESOURCE_CONTROL_REQUIRED = TRUE;

#else

#ifdef DEBUG_LEVEL1
	fprintf(DEBUG_FILE1, "ir_actual_check_init_request: ACCESS/RESOURCE CONTROL IS NOT ENFORCED\n");
#endif

    /*
     * if the client is a SR system and the access and resource control are
     * not enforced, downgrade to SR.
     */

    if ((((PDU_ptr->a.initReq->protocolVersion.bits[0] & SR_PROTOCOL_BIT) == SR_PROTOCOL_BIT) && ((PDU_ptr->a.initReq->protocolVersion.bits[0] & IR_PROTOCOL_BIT) == 0)) ||
            (((PDU_ptr->a.initReq->options.bits[0] & ACCESS_CONTROL_BIT) == 0) || ((PDU_ptr->a.initReq->options.bits[0] & RESOURCE_CONTROL_BIT) == 0))
        )
    {
        /* downgrade to SR */
        *protocol_version = PDU_ptr->a.initReq->protocolVersion.bits[0];
        *options = PDU_ptr->a.initReq->options.bits[0];
        ACCESS_CONTROL_REQUIRED = FALSE;
        RESOURCE_CONTROL_REQUIRED = FALSE;

#ifdef DEBUG_LEVEL1
        fprintf(DEBUG_FILE1, "ir_actual_check_init_request: down grade to SR Protocol\n");
#endif

    }
    else if ((PDU_ptr->a.initReq->protocolVersion.bits[0] & IR_PROTOCOL_BIT) == IR_PROTOCOL_BIT)
    {
        /* if IR */
        *protocol_version = (char) 0xc0;        /* SR+IR */
        *options = (char) 0xfe;        /* all on */
        ACCESS_CONTROL_REQUIRED = TRUE;
        RESOURCE_CONTROL_REQUIRED = TRUE;

#ifdef DEBUG_LEVEL1
        fprintf(DEBUG_FILE1, "ir_actual_check_init_request: IR protocol\n");
#endif

    }
    else
    {
        display_error("KERNEL", ERR_INVALID_INIT_PROTOCOL_VERSION, -1);
        return (FAILED);
    }
#endif /* end of #ifdef ENFORCE_ACCESS_RESOURCE_CONTROL */

#ifdef ACTIVE_TEMPORARY
    if (PDU_ptr->a.initReq->preferredMessageSize > MAXMESSAGESIZE)
    {
        display_error("KERNEL", ERR_INVALID_INIT_MESS_SIZE_REQUEST, -1);
        return (FAILED);
    }
#endif

    return (SUCCESS);
}

/*****************************************************************************
 *
 * Function: ir_check_init_request
 *
 * Purpose: This function performs the following functions:
 *              - checks the init request PDU,
 *              - creates and sends the init response APDU to IR kernel if a
 *                bad mandatory field is detected.
 *
 * Input Arguments:
 *              PDU_ireq - init request PDU pointer
 *              data_ptr - temporary data pointer for ASN functions
 *              socketFd - the socket Fd for communicating to IR kernel
 *
 * Return values: SUCCESS or FAILED
 *
 * Output Arguments:
 *
 * External Variables:
 *
 * Function used:
 *              ir_check_init_request()
 *              ir_create_init_response()
 *              cs_send_data()
 *              display_error()
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference:
 *
 * Notes:
 *
 ****************************************************************************/
ir_check_init_request(PDU_ireq, data_ptr, socketFd)
    APDU *PDU_ireq;
	char **data_ptr;
    int socketFd;
{
    int result;
    long APDU_length;
    SBuf APDU_iresp;
    char protocol_version, options;

#ifdef DEBUG_LEVEL1
    fprintf(DEBUG_FILE1, "Calling ir_process_init_request\n");
#endif

    result = ir_actual_check_init_request(PDU_ireq, &protocol_version, &options);

    if (result == FAILED)
	{
		/* rejected init request */
		ir_create_init_response(PDU_ireq, FALSE, protocol_version, options, data_ptr, &APDU_iresp, &APDU_length);

		/* send the init response to client */
		result = cs_send_data(APDU_iresp.dataStart, APDU_length, socketFd);

#ifdef DEBUG_LEVEL1
		if (result == SUCCESS)
			fprintf(DEBUG_FILE1, "successfully sent init response\n");
		else
			fprintf(DEBUG_FILE1, "failed sent init response\n");
#endif

	}

    return (result);
}

/*****************************************************************************
 *
 * Function: ir_check_init_response
 *
 * Purpose: This function checks the init response PDU to make sure the
 *          mandatory fields are present and valid.
 *
 * Input Arguments: PDU_ptr - init request PDU pointer
 *
 * Return values: SUCCESS or FAILED
 *
 * Output Arguments:
 *
 * External Variables:
 *
 * Function used:
 *              display_error()
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference:
 *
 * Notes:
 *
 ****************************************************************************/
ir_check_init_response(PDU_ptr)
    APDU *PDU_ptr;
{

    int i;

#ifdef DEBUG_LEVEL1
    fprintf(DEBUG_FILE1, "Calling ir_check_init_response\n");
#endif

    /* checking the mandatory fields */

#ifdef DEBUG_LEVEL3
    fprintf(DEBUG_FILE3, "ir_check_init_response: protocolVersion.bitLen=%d\n", PDU_ptr->a.initResp->protocolVersion.bitLen);
    fprintf(DEBUG_FILE3, "ir_check_init_response: protocolVersion.bits[0]=%x\n", PDU_ptr->a.initResp->protocolVersion.bits[0] & 0xff);
#endif

    if ((PDU_ptr->a.initResp->protocolVersion.bitLen <= 0) ||
            (PDU_ptr->a.initResp->protocolVersion.bits[0] & 0xff <= 0)
        )
    {
        display_error("KERNEL", ERR_EMPTY_INIT_PROTOCOL_VERSION, -1);
        return (FAILED);
    }

    if (PDU_ptr->a.initResp->preferredMessageSize <= 0)
    {
        display_error("KERNEL", ERR_EMPTY_INIT_PREFERRED_MESSAGE_SIZE, -1);
        return (FAILED);
    }

    if (PDU_ptr->a.initResp->maximumRecordSize <= 0)
    {
        display_error("KERNEL", ERR_EMPTY_INIT_MAXIMUM_RECORD_SIZE, -1);
        return (FAILED);
    }

    if (PDU_ptr->a.initResp->result != TRUE && PDU_ptr->a.initResp->result != FALSE)
    {
        display_error("KERNEL", ERR_EMPTY_INIT_RESP_RESULT, -1);
        return (FAILED);
    }

    /* end of checking the mandatory fields */

    return (SUCCESS);
}
