/****************************************************************************
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *~                             IR Kernel                                    ~
 *~                         Copyright(C) 1993                                ~
 *~                       Software Kinetics Ltd.                             ~
 *~                        All Rights Reserved                               ~
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Copyright Software Kinetics Ltd.
 * 
 * Software Kinetics Ltd. has granted a licence of this software to
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve,
 * reproduce and sell the software.
 *
 * The software is provided to the user "as is" without warranty of
 * any kind.  Neither Software Kinetics Ltd. nor the National Library
 * of Canada warrants,guarantees or makes any representation regarding
 * the use, or the results of the use of the software in terms of
 * accuracy,reliability, or otherwise.  The entire risk as results and
 * performance of the software is assumed by the user. Neither
 * Software Kinetics Ltd. nor the National Library of Canada shall be
 * liable for any direct, indirect, consequential or incidental
 * damages arising out the use of software.
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
 *
 * Program: tpckprnt.c
 *
 * Purpose: This file contains the functions to check the present request PDU
 *          and present response PDU.
 *
 * Invocation:
 *
 * Input Arguments:
 *
 * Output/Exit Values: None
 *
 * Functions used:
 *
 * Version: Date:   94/05/24
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference: IR KERNEL - TARGET PRELIMINARY SOFTWARE DESIGN DOCUMENT
 *            IR KERNEL - TARGET SOFTWARE DETAIL DESIGN DOCUMENT
 *            IR KERNEL - INTERFACE REQUIREMENTS SPECIFICATION
 *            OSI - ANSI Z39.50 DOCUMENTATION
 *
 * Notes:
 *
 * $Id: $ 
 * $Log: $
 *
 *
 *****************************************************************************/

#include <stdio.h>
#include <rpc/rpc.h>
#include <fcntl.h>
#include <pwd.h>
#include <ctype.h>
#include <sys/errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include "ir.h"
#include "ir_oid.h"
#include "asn_incl.h"
#include "irerror.h"
#include "irconst.h"

/**************************** Extern Global Ids **************************/

extern int errno;

/*****************************************************************************
 *
 * Function: ir_fill_present_response
 *
 * Purpose: This function fills and creates the search response PDU with error 
 *          code 8 indicating invalid mandatory fields.
 *
 * Input Arguments:
 *              PDU_preq - present request PDU pointer
 *
 * Return values: SUCCESS or FAILED
 *
 * Output Arguments:
 *              PDU_presp - present response PDU pointer
 *
 * External Variables:
 *
 * Function used:
 *
 * Algorithm:
 *
 * Version: Date:   94/05/24
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference:
 *
 * Notes:
 *
 ****************************************************************************/
ir_fill_present_response(PDU_preq, PDU_presp)
    APDU *PDU_preq;
    APDU *PDU_presp;
{

    char error_str[73];

#ifdef DEBUG_LEVEL1
    fprintf(DEBUG_FILE1, "Calling ir_fill_present_response\n");
#endif

    PDU_presp->a.presentResp = (struct PresentResp *) Asn1Alloc(sizeof(struct PresentResp));

    PDU_presp->choiceId = APDU_PRESENTRESP;

    PDU_presp->a.presentResp->referenceId.octetLen = PDU_preq->a.presentReq->referenceId.octetLen;
    PDU_presp->a.presentResp->referenceId.octs = (char *) Asn1Alloc(PDU_preq->a.presentReq->referenceId.octetLen);
    memcpy(PDU_presp->a.presentResp->referenceId.octs, PDU_preq->a.presentReq->referenceId.octs, PDU_preq->a.presentReq->referenceId.octetLen);

    PDU_presp->a.presentResp->numRecordsReturned = 1L;
    PDU_presp->a.presentResp->nextResultSetPosition = 0L;

    PDU_presp->a.presentResp->presentStatus = (AsnInt *) Asn1Alloc(sizeof(AsnInt));
    *PDU_presp->a.presentResp->presentStatus = 0;

    PDU_presp->a.presentResp->dbOrDiagnosticRec = (struct Records *) Asn1Alloc(sizeof(struct Records));

	*PDU_presp->a.presentResp->presentStatus = FAILURE_4;

	/* create diagnostics record (error record) */
	PDU_presp->a.presentResp->dbOrDiagnosticRec->a.nonSurDiagnostics = (struct DiagRec *) Asn1Alloc(sizeof(struct DiagRec));

	PDU_presp->a.presentResp->dbOrDiagnosticRec->choiceId = RECORDS_NONSURDIAGNOSTICS;

	PDU_presp->a.presentResp->dbOrDiagnosticRec->a.nonSurDiagnostics->diagnosticSetId.octetLen = strlen(OID_DIAGNOSTIC_SET_BIB1);
	PDU_presp->a.presentResp->dbOrDiagnosticRec->a.nonSurDiagnostics->diagnosticSetId.octs = (char *) Asn1Alloc(PDU_presp->a.presentResp->dbOrDiagnosticRec->a.nonSurDiagnostics->diagnosticSetId.octetLen);
	memcpy(PDU_presp->a.presentResp->dbOrDiagnosticRec->a.nonSurDiagnostics->diagnosticSetId.octs, OID_DIAGNOSTIC_SET_BIB1, PDU_presp->a.presentResp->dbOrDiagnosticRec->a.nonSurDiagnostics->diagnosticSetId.octetLen);

	PDU_presp->a.presentResp->dbOrDiagnosticRec->a.nonSurDiagnostics->condition = 8; /* error code */

	strcpy(error_str, "THE MANDATORY FIELDS ARE INVALID");

	PDU_presp->a.presentResp->dbOrDiagnosticRec->a.nonSurDiagnostics->addInfo.octetLen = strlen(error_str);
	PDU_presp->a.presentResp->dbOrDiagnosticRec->a.nonSurDiagnostics->addInfo.octs = (char *) Asn1Alloc(PDU_presp->a.presentResp->dbOrDiagnosticRec->a.nonSurDiagnostics->addInfo.octetLen);
	memcpy(PDU_presp->a.presentResp->dbOrDiagnosticRec->a.nonSurDiagnostics->addInfo.octs, error_str, PDU_presp->a.presentResp->dbOrDiagnosticRec->a.nonSurDiagnostics->addInfo.octetLen);

    return (SUCCESS);
}

/*****************************************************************************
 *
 * Function: ir_create_present_response
 *
 * Purpose: This function creates the present response PDU, encodes, and
 *          creates the present response APDU for sending to client.
 *
 * Input Arguments:
 *          PDU_preq - present request PDU
 *          data_ptr - temporary data buffer for encoding process
 *
 * Return values: SUCCESS or FAILED
 *
 * Output Arguments:
 *              outputBuf - encoded APDU pointer
 *              APDU_length - the size of the encoded APDU
 *
 * External Variables:
 *
 * Function used:
 *              ir_fill_present_response()
 *              SBufInstallData()
 *              SBufResetInWriteRvsMode()
 *              BEncAPDU()
 *              ir_fwrite()
 *
 * Algorithm:
 *
 * Version: Date:   94/05/24
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference:
 *
 * Notes:
 *
 ****************************************************************************/
ir_create_present_response(PDU_preq, data_ptr, outputBuf, APDU_length)
    APDU *PDU_preq;
    char **data_ptr;
    SBuf *outputBuf;
    int *APDU_length;
{
    APDU PDU_ptr;
    int i;

#ifdef DEBUG_LEVEL1
    fprintf(DEBUG_FILE1, "Calling ir_create_present_response()\n");
#endif

    ir_fill_present_response(PDU_preq, &PDU_ptr);

#ifdef DEBUG_LEVEL2
    fprintf(DEBUG_FILE2, "ir_create_present_response: begin dump data:\n");
    PrintPresentResp(DEBUG_FILE2, PDU_ptr.a.presentResp, 0);
    fprintf(DEBUG_FILE2, "ir_create_present_response: end dump data:\n");
#endif

    SBufInstallData(outputBuf, (*data_ptr), MAXMESSAGESIZE * 2);
    SBufResetInWriteRvsMode(outputBuf);

    *APDU_length = BEncAPDU(outputBuf, &PDU_ptr);

	if (SBufWriteError(outputBuf))
	{
		display_error("KERNEL", ERR_ENCODE_APDU, -1);
		return(FAILED);
	}

#ifdef DEBUG_LEVEL2
    fprintf(DEBUG_FILE2, "ir_create_present_response: Present Response APDU Dump from tppresnt\n");
    ir_fwrite(SBufDataPtr(outputBuf), SBufDataLen(outputBuf), 1, DEBUG_FILE2);
#endif

    return (SUCCESS);
}

/*****************************************************************************
 *
 * Function: ir_actual_check_present_request
 *
 * Purpose: This function checks the present request PDU to make sure the
 *          mandatory fields are present and valid.
 *
 * Input Arguments: PDU_ptr - present request PDU pointer
 *
 * Return values: SUCCESS or FAILED
 *
 * Output Arguments:
 *
 * External Variables:
 *
 * Function used:
 *              display_error()
 *
 * Algorithm:
 *
 * Version: Date:   94/05/24
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference:
 *
 * Notes:
 *
 ****************************************************************************/
ir_actual_check_present_request(PDU_ptr)
    APDU *PDU_ptr;
{

#ifdef DEBUG_LEVEL1
        fprintf(DEBUG_FILE1, "Calling ir_actual_check_present_request\n");
#endif

    if (PDU_ptr->a.presentReq->resultSetId.octetLen <= 0 ||
            PDU_ptr->a.presentReq->resultSetId.octs == NULL)
    {
#ifdef DEBUG_LEVEL1
        fprintf(DEBUG_FILE1, "ir_actual_check_present_request: ERR_EMPTY_PRESENT_RESULT_SET_ID\n");
#endif
        display_error("KERNEL", ERR_EMPTY_PRESENT_RESULT_SET_ID, -1);
        return (FAILED);
    }

    if (PDU_ptr->a.presentReq->resultSetStartPoint < 0)
    {
#ifdef DEBUG_LEVEL1
        fprintf(DEBUG_FILE1, "ir_actual_check_present_request: ERR_EMPTY_PRESENT_RESULT_SET_START_POINT\n");
#endif
        display_error("KERNEL", ERR_EMPTY_PRESENT_RESULT_SET_START_POINT, -1);
        return (FAILED);
    }

    if (PDU_ptr->a.presentReq->numRecordsRequested <= 0)
    {
#ifdef DEBUG_LEVEL1
        fprintf(DEBUG_FILE1, "ir_actual_check_present_request: ERR_EMPTY_PRESENT_NUMBER_RECORDS_REQUESTED\n");
#endif
        display_error("KERNEL", ERR_EMPTY_PRESENT_NUMBER_RECORDS_REQUESTED, -1);
        return (FAILED);
    }

    return (SUCCESS);
}

/*****************************************************************************
 *
 * Function: ir_check_present_request
 *
 * Purpose: This function performs the following functions:
 *              - checks the present request PDU,
 *              - creates and sends the present response APDU to client if
 *                any mandatory fields is invalid.
 *
 * Input Arguments:
 *              PDU_preq - present request PDU pointer
 *              data_ptr - temporary data pointer for ASN functions
 *              socketFd - the socket Fd for communicating to IR kernel
 *
 * Return values: SUCCESS or FAILED
 *
 * Output Arguments:
 *
 * External Variables:
 *
 * Function used:
 *              ir_check_present_request()
 *              ir_create_present_response()
 *              tpa_send_APDU()
 *              display_error()
 *
 * Algorithm:
 *
 * Version: Date:   94/05/24
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference:
 *
 * Notes:
 *
 ****************************************************************************/
ir_check_present_request(PDU_preq, data_ptr, socketFd)
    APDU *PDU_preq;
    char **data_ptr;
    int socketFd;
{
    int result;
    long APDU_length;
    SBuf APDU_presp;

#ifdef DEBUG_LEVEL1
    fprintf(DEBUG_FILE1, "Calling ir_check_present_request\n");
#endif

    result = ir_actual_check_present_request(PDU_preq);

    if (result == FAILED)
    {
        ir_create_present_response(PDU_preq, data_ptr, &APDU_presp, &APDU_length);

		if (cs_send_data(APDU_presp.dataStart, (int) APDU_length, socketFd) == FAILED)
		{
			display_error("KERNEL", ERR_SEND_DATA_TO_CLIENT, errno);
			return (FAILED);
		}
	}

    return (SUCCESS);
}

/*****************************************************************************
 *
 * Function: ir_check_present_response
 *
 * Purpose: This function checks the mandatory fields of present response PDU 
 *          for validity.
 *
 * Input Arguments: PDU_ptr - present response PDU pointer
 *
 * Return values: SUCCESS or FAILED
 *
 * Output Arguments:
 *
 * External Variables:
 *
 * Function used:
 *              display_error()
 *
 * Algorithm:
 *
 * Version: Date:   94/05/24
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference:
 *
 * Notes:
 *
 ****************************************************************************/
ir_check_present_response(PDU_ptr)
    APDU *PDU_ptr;
{

    if (PDU_ptr->a.presentResp->numRecordsReturned < 0)
    {
        display_error("KERNEL", ERR_EMPTY_PRESENT_RESP_NO_OF_RECS_RETURN, -1);
        return (FAILED);
    }

    if (PDU_ptr->a.presentResp->nextResultSetPosition < 0)
    {
        display_error("KERNEL", ERR_EMPTY_PRESENT_RESP_NEXT_RESULT_SET_POSITION, -1);
        return (FAILED);
    }

    if (*PDU_ptr->a.presentResp->presentStatus < 0 || *PDU_ptr->a.presentResp->presentStatus > 5)
    {
        display_error("KERNEL", ERR_EMPTY_PRESENT_RESP_PRESENT_STATUS, -1);
        return (FAILED);
    }

    return (SUCCESS);
}
